@@ -34,13 +34,11 @@ extern void detach __((void));
 /* die.c */
 extern void die __((int status, char *message));
 
-#ifdef USE_INET
 /* dottedquad.c */
-#ifndef	IPPROTO_IP
+#ifdef HAVE_NETINET_IN_H
 # include <netinet/in.h>
 #endif
 extern char *dottedquad __((struct in_addr *inp));
-#endif
 
 /* esyslib.c */
 extern int eopen __((char *s, int f, int m));
