@@ -42,11 +42,11 @@ extern int wait();
 # endif
 #endif
 
-#ifndef WIFEXITED
-# define WIFEXITED(s) (((s) >> 8) & 0377)
+#ifndef WEXITSTATUS
+# define WEXITSTATUS(s) (((s) >> 8) & 0377)
 #endif
-#ifndef WIFSIGNALED
-# define WIFSIGNALED(s) ((s) & 0177)
+#ifndef WSIGNALSTATUS
+# define WSIGNALSTATUS(s) ((s) & 0177)
 #endif
 
 #ifndef	SIGCHLD
@@ -575,16 +575,16 @@ execute(c, caller, oretcode, name)
 			trapexit(retcode);
 		return retcode;
 	}
-	if (WIFSIGNALED(status) > 0) {
-		if (WIFSIGNALED(status) != SIGINT) {
-			fprintf(stderr, "%s", strsignal(WIFSIGNALED(status)));
+	if (WSIGNALSTATUS(status) != 0) {
+		if (WSIGNALSTATUS(status) != SIGINT) {
+			fprintf(stderr, "%s", strsignal(WSIGNALSTATUS(status)));
 			if (status&0200)
 				fprintf(stderr, CORE_DUMPED);
 		}
 		fprintf(stderr, "\n");
-		retcode = 0200 + WIFSIGNALED(status);
+		retcode = 0200 + WSIGNALSTATUS(status);
 	} else
-		retcode = WIFEXITED(status);
+		retcode = WEXITSTATUS(status);
 	if (isset('t'))
 		trapexit(retcode);
 	return retcode;
