@@ -10,11 +10,26 @@
  * using IFS characters is also done here.
  */
 
-#include <stdio.h>
 #include "hostenv.h"
+#include <stdio.h>
 #include <sys/file.h>
 #include <sys/stat.h>
-#include NDIR_H
+
+#ifdef HAVE_DIRENT_H
+# include <dirent.h>
+#else /* not HAVE_DIRENT_H */
+# define dirent direct
+# ifdef HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif /* HAVE_SYS_NDIR_H */
+# ifdef HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif /* HAVE_SYS_DIR_H */
+# ifdef HAVE_NDIR_H
+#  include <ndir.h>
+# endif /* HAVE_NDIR_H */
+#endif /* HAVE_DIRENT_H */
+
 #include "sh.h"
 #include "flags.h"
 #include "malloc.h"
@@ -62,13 +77,13 @@ struct sawpath {
  * stat() since in that case a super-glob can lead to infinite recursion.
  */
 
-#ifdef	USE_LSTAT
+#ifdef	HAVE_LSTAT
 extern int lstat();
 int (*statfcn)() = lstat;	/* stat if following symlinks, o/w lstat */
-#else	/* !USE_LSTAT */
+#else	/* !HAVE_LSTAT */
 extern int stat();
 int (*statfcn)() = stat;
-#endif	/* USE_LSTAT */
+#endif	/* HAVE_LSTAT */
 
 /*
  * Qsort() comparison function to sort pathnames.
@@ -191,7 +206,7 @@ glut(cwd, pwd, bp, recur, swp)
 		namlen,		/* filename length */
 		i;
 	struct stat stbuf;
-	struct NDIR_TYPE *dp;
+	struct dirent *dp;
 	DIR *dirp;
 	
 	if (interrupted)
