@@ -26,27 +26,6 @@
 #include "shconfig.h"
 
 
-#ifdef  HAVE_WAITPID
-# include <sys/wait.h>
-#else
-# ifdef HAVE_WAIT3
-#  include <sys/wait.h> /* Has BSD wait3() */
-# else
-#  ifdef HAVE_SYS_WAIT_H /* POSIX.1 compatible */
-#   include <sys/wait.h>
-#  else /* Not POSIX.1 compatible, lets fake it.. */
-extern int wait();
-#  endif
-# endif
-#endif
-
-#ifndef WIFEXITED
-# define WIFEXITED(s) (((s) >> 8) & 0377)
-#endif
-#ifndef WIFSIGNALED
-# define WIFSIGNALED(s) ((s) & 0177)
-#endif
-
 int magic_number = 3;		/* check id for precompiled script files */
 long bin_magic   = 1;		/* Another check-id -- exec file  st_ctime ? */
 
