@@ -111,6 +111,6 @@ extern int	(*std_printf)();
 extern int	std_fread(), std_fwrite();
 extern long	std_ftell();
 extern int	sioprintf(), siofprintf(), siofscanf();
-extern volatile int siomore();
+extern int	siomore();
 #endif	/* REALSTDIO */
 #endif	/* Z_IO_H */
