@@ -16,7 +16,22 @@
 #include <pwd.h>			/* for run_homedir() */
 #include <grp.h>			/* for run_grpmems() */
 #include <errno.h>
-#include NDIR_H
+
+#ifdef HAVE_DIRENT_H
+# include <dirent.h>
+#else /* not HAVE_DIRENT_H */
+# define dirent direct
+# ifdef HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif /* HAVE_SYS_NDIR_H */
+# ifdef HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif /* HAVE_SYS_DIR_H */
+# ifdef HAVE_NDIR_H
+#  include <ndir.h>
+# endif /* HAVE_NDIR_H */
+#endif /* HAVE_DIRENT_H */
+
 #include "zmsignal.h"
 #include "mail.h"
 #include "interpret.h"
@@ -451,7 +466,7 @@ run_cadddr(avl, il)
 	return car(il);
 }
 
-static SIGNAL_TYPE
+static RETSIGTYPE
 sig_exit(sig)
 int sig;
 {
@@ -467,7 +482,7 @@ int sig;
 
 static int gothup = 0;
 
-SIGNAL_TYPE
+RETSIGTYPE
 sig_hup(sigarg)
 int sigarg;
 {
@@ -655,7 +670,7 @@ rd_stability(dirp,dirs)
 	register int i;
 	int deindex, nbindex, did_cnt;
 	int namelen;
-	struct NDIR_TYPE *dp;
+	struct dirent *dp;
 	struct stat statbuf;
 	char pathbuf[512]; /* Enough ? */
 
@@ -732,7 +747,7 @@ rd_instability(dirp, dirs)
 	DIR *dirp;
 	char *dirs;
 {
-	struct NDIR_TYPE *dp;
+	struct dirent *dp;
 	int did_cnt = 0;
 	struct stat statbuf;
 	char pathbuf[512];
@@ -1936,37 +1951,17 @@ run_runas(argc, argv)
 	if (!initeduid)
 		myuid = geteuid();
 	if (myuid == 0 && uid != 0) {
-		if (
-#ifdef	USE_SETREUID
-		    setreuid(0, uid)
-#else	/* SVID ? */
-#ifdef	USE_SETEUID
-		    seteuid(uid)
-#else
-		    setuid(uid)	/* Only reversible under recent SVID! */
-#endif	/* !USE_SETEUID */
-#endif	/* !USE_SETREUID */
-			< 0) {
+		if (setreuid(0, uid) < 0) {
 			fprintf(stderr, "%s: setuid(%d): %s\n",
-					       argv[0], uid, strerror(errno));
+				argv[0], uid, strerror(errno));
 			return 1;
 		}
 	}
 	/* must be builtin or defined function */
 	r = s_apply(argc - 2, argv + 2);
 	if (myuid == 0 && uid != 0) {
-	  if (
-#ifdef	USE_SETREUID
-	      setreuid(0, 0)
-#else	/* SVID ? */
-#ifdef	USE_SETEUID
-	      seteuid(0)
-#else
-	      setuid(0)	/* Only works under recent SVID! */
-#endif	/* !USE_SETEUID */
-#endif	/* !USE_SETREUID */
-	       < 0)
-		  abort();
+	  if (setreuid(0, 0) < 0)
+	    abort();
 	}
 
 	return r;
