@@ -26,11 +26,11 @@
 		  sys/stream.h, which has DIFFERENT "NOERROR" in it.. */
 #endif
 
-#ifdef linux
+#ifdef __linux__
 #define __USE_BSD 1	/* Linux headers ... Auch..  <endian.h> */
 #endif
-#include <arpa/nameser.h>
-#include <resolv.h>
+#include "arpa/nameser.h"
+#include "resolv.h"
 #include "search.h"
 
 #ifdef	__alpha		/* The IP-address processing must be changed to
@@ -57,7 +57,7 @@
 
 #ifndef	BIND_VER
 #ifdef	GETLONG
-/* 4.7.3 introduced the {GET,PUT}{LONG,SHORT} macros in nameser.h */
+/* 4.7.3 introduced the {GET,PUT}{LONG,SHORT} macros in arpa/nameser.h */
 #define	BIND_VER	473
 #else	/* !GETLONG */
 #define	BIND_VER	472
@@ -407,7 +407,7 @@ getmxrr(host, localhost, ttlp, depth)
 		cp += n;
 		mx[nmx].host = (msgdata *)strsave((char*)hbuf);
 		if (localhost != NULL && CISTREQ(hbuf, localhost))
-		    if (maxpref < 0 || maxpref > mx[nmx].pref)
+		    if ((maxpref < 0) || (maxpref > (int)mx[nmx].pref))
 			maxpref = mx[nmx].pref;
 		++nmx;
 	}
@@ -420,7 +420,7 @@ getmxrr(host, localhost, ttlp, depth)
 	/* discard MX RRs with a value >= that of localdomain */
 	if (maxpref >= 0) {
 		for (n = 0; n < nmx; ++n) {
-			if (mx[n].pref >= maxpref && mx[n].host != NULL) {
+			if (((int)mx[n].pref >= maxpref) && (mx[n].host != NULL)) {
 				free((char *)mx[n].host);
 				mx[n].host = NULL;
 			}
