@@ -9,7 +9,7 @@
 #include <ctype.h>
 #include <sys/file.h>
 #include "search.h"
-#ifdef	USE_MMAP
+#ifdef	HAVE_MMAP
 #include <sys/mman.h>
 #endif
 #include <errno.h>
@@ -18,7 +18,7 @@ extern FILE *fpmap();
 extern int deferit, cistrcmp();
 extern void v_set();
 
-#ifdef USE_MMAP
+#ifdef HAVE_MMAP
 extern void seq_remap __((struct file_map *, long newsize));
 extern char *mfgets __((char *, int, struct file_map *));
 #endif
@@ -41,7 +41,7 @@ search_bin(sip)
 	struct spblk *spl;
 	char buf[BUFSIZ];
 	struct file_map *fm;
-#ifdef	USE_MMAP
+#ifdef	HAVE_MMAP
 	struct stat fst;
 #endif
 
@@ -62,7 +62,7 @@ reopen:
 		}
 		fm = (struct file_map *)emalloc(sizeof(struct file_map));
 		fm->fp = fp;
-#ifdef	USE_MMAP
+#ifdef	HAVE_MMAP
 		fstat(FILENO(fp),&fst);
 		fm->size = fst.st_size;
 		fm->mtime = fst.st_mtime;
@@ -96,7 +96,7 @@ reopen:
 			spl->data = (u_char *)fm;
 	}
 	fp = fm->fp;
-#ifdef	USE_MMAP
+#ifdef	HAVE_MMAP
 	/* This  fstat()  for possible seq_remap() trigger causes a bit
 	   more syscalls, than is really necessary.   Therefore it is
 	   likely best to have "-m" option on the relation definitions
@@ -105,7 +105,6 @@ reopen:
 	   possible upper-level "-m"..					*/
 #define NO_SEQREMAP
 #ifndef NO_SEQREMAP
-	if (fstat(FILENO(fp),&fst) < 0) abort(); /* Will succeed, or crash.. */
 	if (fstat(FILENO(fp),&fst) < 0) abort(); /* Will succeed, or crash.. */
 	if (fst.st_mtime != fm->mtime ||
 	    fst.st_size  != fm->size) {
@@ -165,7 +164,7 @@ reopen:
 	}
 #endif
 
-#ifndef	USE_MMAP
+#ifndef	HAVE_MMAP
 	bot = 0;
 	fseek(fp, 0L, 2);	/* EOF */
 	top = ftell(fp);
