@@ -65,7 +65,7 @@ extern int	tolower __((const int));
 #endif
 
 /* File: conf.c */
-extern char	 files_gid;
+extern int	 files_gid;
 extern char	*cf_suffix;
 extern char	*default_trusted[];
 extern char	*err_prio_list[];
@@ -137,7 +137,7 @@ extern int	isRecpntAddr;
 extern void	squirrel __((struct envelope *e, char *text));
 extern struct header	*erraddress __((struct envelope *e));
 extern void	reject __((struct envelope *e, char *msgfile));
-extern void	defer __((struct envelope *e));
+extern void	defer __((struct envelope *e, char *why));
 extern struct header	*mkSender __((struct envelope *e, u_char *name, int flag));
 extern struct header	*mkTrace __((struct envelope *e));
 extern int	nullhost __((u_char *s));
