@@ -105,20 +105,20 @@ run_rfc822(argc, argv)
 		fprintf(stderr, "router: cannot open %s\n", file);
 		status = PERR_BADOPEN;
 	} else {
-/* XX: DEBUG STUFF! */
-if (FILENO(e->e_fp) < 3) {
-  fprintf(stderr,"RFC822: While opening mail-file '%s', got fd=%d  AARGH! (fds 0..2 should never close on us!)",
-	  file, FILENO(e->e_fp));
-  fflush(stderr);
-  /* abort(); */
-}
+	  /* XX: DEBUG STUFF! */
+	  if (FILENO(e->e_fp) < 3) {
+	    fprintf(stderr,"RFC822: While opening mail-file '%s', got fd=%d  AARGH! (fds 0..2 should never close on us!)",
+		    file, FILENO(e->e_fp));
+	    fflush(stderr);
+	    /* abort(); */
+	  }
 		status = PERR_OK;
 		setvbuf(e->e_fp, buf, _IOFBF, sizeof buf);
 		e->e_file = file;
 		status = makeLetter(e, 0);
 	}
 
-	if (status == 0)
+	if (status == PERR_OK)
 		/* passing the file is redundant but nice for backtraces */
 		status = sequencer(e, file);
 
@@ -127,20 +127,22 @@ if (FILENO(e->e_fp) < 3) {
 		break;
 	case PERR_BADOPEN:
 		break;
+#if 0 /* not used */
 	case PERR_BADCONTINUATION:	/* fatal */
-		squirrel(e, "continuation line prior to header");
+		defer(e, "continuation line prior to header");
 		break;
 	case PERR_BADSUBMIT:		/* fatal */
-		squirrel(e, "unrecognized envelope information");
+		defer(e, "unrecognized envelope information");
 		break;
+#endif
 	case PERR_LOOP:			/* fatal */
-		squirrel(e, "possible looping message");
+		reject(e, "loopexceeded");
 		break;
 	case PERR_ENVELOPE:		/* fatal */
 		reject(e, "envelope");
 		break;
 	case PERR_DEFERRED:
-		defer(e);
+		defer(e, "deferred");
 		break;
 	case PERR_HEADER:		/* fatal */
 		reject(e, "header");
@@ -149,10 +151,10 @@ if (FILENO(e->e_fp) < 3) {
 		reject(e, "norecipients");
 		break;
 	case PERR_NOSENDER:
-		squirrel(e, "really truly Unknown Sender!");
+		squirrel(e, "really truly Unknown Sender");
 		break;
 	case PERR_CTRLFILE:
-		defer(e);
+		defer(e, "ctrlfile");
 		break;
 	default:
 		abort();
@@ -250,12 +252,15 @@ makeLetter(e, octothorp)
 				t->t_next = makeToken(linebuf, n-1);
 				t->t_next->t_type = Line;
 			} else if (!octothorp) {
-				/* return PERR_BADCONTINUATION */ ;
+				return PERR_BADCONTINUATION;
 				badcontinuationflag = 1;
 			}
 		} else if (!inheader		/* envelope information */
 			   && (*(cp=linebuf-i) == ' ' || *cp == '\t'
 				|| *cp == '\n')) {
+			struct headerinfo *hd;
+			HeaderSemantics osem;
+
 			if (octothorp && linebuf[0] == '#')
 				continue;
 			/* cons up a new one at top of envelope header list */
@@ -265,20 +270,18 @@ makeLetter(e, octothorp)
 			h->h_lines = makeToken(linebuf-i+1, n > 1-i ? n+i-2: 0);
 			h->h_lines->t_type = Line;
 			switch (h->h_descriptor->class) {
-				struct headerinfo *hd;
-				HeaderSemantics osem;
 
 			case normal:
 			case Resent:
 				/* error */
 				fprintf(stderr,
-					"%s: unknown envelope header: ",
-					progname);
+					"%s: unknown envelope header (class %d): ",
+					progname, h->h_descriptor->class);
 				fwrite((char *)linebuf, sizeof (char), -i, stderr);
 				putc('\n', stderr);
 				h->h_contents = hdr_scanparse(e, h, octothorp);
 				h->h_stamp = hdr_type(h);
-				/* return PERR_BADSUBMIT; */
+				return PERR_BADSUBMIT;
 				badsubmitflag = 1;
 				break;
 			case eFrom:		/* pity to break the elegance */
@@ -289,6 +292,8 @@ makeLetter(e, octothorp)
 				h->h_stamp = hdr_type(h);
 				hd->semantics = osem;
 				break;
+			case eEnvEnd:		/* more elegance breaks */
+				inheader = 1;
 			default:		/* a real envelope header */
 				h->h_contents = hdr_scanparse(e, h, octothorp);
 				h->h_stamp = hdr_type(h);
@@ -325,7 +330,7 @@ makeLetter(e, octothorp)
 	if (badsubmitflag || badcontinuationflag)
 	  squirrel(e, "unrecognized envelope information; entries discarded!");
 
-	return 0;
+	return PERR_OK;
 }
 
 void
@@ -466,9 +471,9 @@ squirrel(e, text)
 	char *path;
 
 #ifndef	USE_ALLOCA
-	path = emalloc(5+strlen(POSTMANDIR)+10);
+	path = emalloc(5+sizeof(POSTMANDIR)+10);
 #else
-	path = alloca(5+strlen(POSTMANDIR)+10);
+	path = alloca(5+sizeof(POSTMANDIR)+10);
 #endif
 	sprintf(path, "../%s/%d", POSTMANDIR, e->e_statbuf.st_ino);
 
@@ -479,9 +484,9 @@ squirrel(e, text)
 	if (elink(e->e_file, path) < 0) {
 		sprintf(path, "..%d", e->e_statbuf.st_ino);
 		erename(e->e_file, path);
-	} else {
-		eunlink(e->e_file);
 	}
+	/* The successfull squirrel MUST NOT delete the original message! */
+
 	fprintf(stderr, "squirrel: %d saved for inspection: %s\n",
 			e->e_statbuf.st_ino, text);
 #ifdef	LOG_ERR
@@ -634,14 +639,12 @@ reject(e, msgfile)
 	while ((n = fread(buf, 1, sizeof buf, e->e_fp)) > 0) {
 		if (fwrite(buf, 1, n, mfp) != n) {
 			squirrel(e, "Couldn't include rejected input message");
-			fclose(fp);
 			mail_abort(mfp);
 			return;
 		}
 	}
 	if (mail_close(mfp) < 0) {
 		squirrel(e, "Couldn't close reject message file");
-		fclose(fp);
 		mail_abort(mfp);
 	}
 	eunlink(e->e_file);
@@ -650,30 +653,42 @@ reject(e, msgfile)
 /* Save a message file away because something went wrong during routing */
 
 void
-defer(e)
+defer(e, why)
 	struct envelope *e;
+	char *why;
 {
 	int i;
-	char *path;
+	char *path, *s;
+	struct stat stbuf;
 
 	if (e->e_fp != NULL && files_gid >= 0) {
 		fchown(FILENO(e->e_fp), -1, files_gid);
 		fchmod(FILENO(e->e_fp),
 			(0440|e->e_statbuf.st_mode) & 0660);
+		fstat(fileno(e->e_fp),&stbuf);
 	}
+	if (stbuf.st_ino > 0) {
 #ifdef	USE_ALLOCA
-	path = alloca(5+strlen(DEFERREDDIR)+strlen(e->e_file));
+	  path = alloca(5+sizeof(DEFERREDDIR)+6+strlen(why));
 #else
-	path = emalloc(5+strlen(DEFERREDDIR)+strlen(e->e_file));
+	  path = emalloc(5+sizeof(DEFERREDDIR)+6+strlen(why));
 #endif
-	sprintf(path, "../%s/%s", DEFERREDDIR, e->e_file);
+	  sprintf(path,"../%s/%d.%s", DEFERREDDIR, stbuf.st_ino, why);
+	} else {
+#ifdef	USE_ALLOCA
+	  path = alloca(5+sizeof(DEFERREDDIR)+strlen(e->e_file)+2+strlen(why));
+#else
+	  path = emalloc(5+sizeof(DEFERREDDIR)+strlen(e->e_file)+2+strlen(why));
+#endif
+	  sprintf(path, "../%s/%s.%s", DEFERREDDIR, e->e_file, why);
+	}
+	s = path;
+	while (s = strchr(s, ' ')) /* remap blanks to '-':es.. */
+	  *s = '-';
 	unlink(path);
 
-	/* XX: Deferrance uses  link()/unlink(), while rename() is equally
-	       valid choice */
-
-	/* try linking every few seconds for a minute */
-	for (i = 0; elink(e->e_file, path) < 0 && i < 10 && errno != ENOENT;++i)
+	/* try renameing every few seconds for a minute */
+	for (i = 0; erename(e->e_file, path) < 0 && i < 10 && errno != ENOENT;++i)
 		sleep(6);
 	if (i >= 10) {
 #ifdef	LOG_ALERT
@@ -682,8 +697,6 @@ defer(e)
 		fprintf(stderr, "cannot defer %s\n", e->e_file);
 		/* XX: make sure file is not unlink()ed in main() */
 	} else {
-		if (!savefile)
-			eunlink(e->e_file);
 #ifdef	LOG_NOTICE
 		syslog(LOG_NOTICE, "%s; %sing deferred",
 			   e->e_messageid == NULL ? e->e_file : e->e_messageid,
@@ -1072,12 +1085,10 @@ sequencer(e, file)
 			&& CISTREQ(h->h_pname, "Received"))) {
 			if (++i >= maxReceived) {
 				fprintf(stderr,
-				     "%s: looping message, squirreled away!\n",
+				     "%s: looping message, rejected!\n",
 					progname);
 				perr = PERR_LOOP;
-				squirrel(e,"looping message, exceeded maxReceived");
-				/* XXX: Should do "loop-count-exceeded"! */
-				break;
+				return perr;
 			}
 		}
 	}
@@ -1105,9 +1116,11 @@ sequencer(e, file)
 		 * the message when the problem is fixed, so we don't tell
 		 * the originator that anything went wrong.
 		 */
-		defer(e);
+#if 0 /* -- rfc822 processing does a reject() on this! */
+		defer(e, "envelope");
 		/* Often the defer() becomes resubmited via cron, but lets
 		   hope the postmaster will see things manually.. */
+#endif
 		return PERR_ENVELOPE;
 	}
 	e->e_from_trusted = makequad();
