@@ -63,9 +63,9 @@ n_apply(cpp, argc, argv)
 {
 	int retval;
 
-	sb_external(fileno(stdout));	/* set up for retrieval of stdout */
+	sb_external(FILENO(stdout));	/* set up for retrieval of stdout */
 	retval = apply(argc, argv);
-	*cpp = sb_retrieve(fileno(stdout));	/* safe alloc'ed memory */
+	*cpp = sb_retrieve(FILENO(stdout));	/* safe alloc'ed memory */
 	return retval;
 }
 
@@ -250,7 +250,7 @@ hdr_rewrite(name, h)
 			s_free_tree(s_value);
 			s_value = NULL;
 			/* t = HDR_SCANNER(l->string); */
-			t = scan822(&l->string, strlen(l->string),
+			t = scan822(&l->string, strlen((char*)(l->string)),
 				    (u_char)'!', (u_char)'%',
 				    0, &ap->a_tokens->p_tokens);
 			/* X: check for errors! */
