@@ -19,15 +19,30 @@
 #include "mail.h"
 #include "scheduler.h"
 #include "malloc.h"
-#include NDIR_H
 
-#if	defined(USE_INET) && (defined(USE_HOSTS) || defined(USE_RESOLV) || defined(USE_YP))
+#ifdef HAVE_DIRENT_H
+# include <dirent.h>
+#else /* not HAVE_DIRENT_H */
+# define dirent direct
+# ifdef HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif /* HAVE_SYS_NDIR_H */
+# ifdef HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif /* HAVE_SYS_DIR_H */
+# ifdef HAVE_NDIR_H
+#  include <ndir.h>
+# endif /* HAVE_NDIR_H */
+#endif /* HAVE_DIRENT_H */
+
+
+#if	defined(HAVE_SOCKET) && (defined(HAVE_RESOLVER) || defined(HAVE_YP))
 #include <netdb.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <fcntl.h>
 #include <sys/file.h>
-#endif	/* USE_INET */
+#endif	/* HAVE_SOCKET */
 
 #ifdef	MALLOC_TRACE
 struct conshell *envarlist = NULL;
@@ -380,7 +395,7 @@ checkrouter()
 	int pid, n, r;
 	FILE *fp;
 	struct stat pidbuf, corebuf;
-	struct NDIR_TYPE *dp;
+	struct dirent *dp;
 	DIR *dirp;
 
 	if (postoffice == NULL)
@@ -467,7 +482,7 @@ checkscheduler()
 {
 	int pid, n, tn, r, sawcore = 0;
 	FILE *fp;
-	struct NDIR_TYPE *dp;
+	struct dirent *dp;
 	DIR *dirp;
 
 	if (postoffice == NULL)
@@ -527,7 +542,7 @@ checkscheduler()
 	default:
 	  printf("never started");
 	  if (n > 0)
-	    printf(" \"%s/%s\" polluted", postoffice, SCHEDULERDIR);
+	    printf(" \"%s/%s\" polluted", postoffice, TRANSPORTDIR);
 	  break;
 	}
 	if (sawcore)
