@@ -4,7 +4,7 @@
  */
 /*
  *	Lots of modifications (new guts, more or less..) by
- *	Matti Aarnio <mea@nic.funet.fi>  (copyright) 1992-1995
+ *	Matti Aarnio <mea@nic.funet.fi>  (copyright) 1992-1996
  */
 
 #include "hostenv.h"
@@ -15,11 +15,10 @@
 #include <fcntl.h>
 #include "mail.h"
 #include "scheduler.h"
-#ifdef	USE_SYSLOG
+#ifdef	HAVE_SYSLOG_H
 #include <syslog.h>
-#endif	/* USE_SYSLOG */
+#endif	/* HAVE_SYSLOG_H */
 #include "ta.h"
-#include NDIR_H
 
 #include "prototypes.h"
 
@@ -180,10 +179,10 @@ FILE *mfp;
 	  s = strrchr(buf,'>');
 	  if (s) *s = 0;
 	  else return; /* No trailing '>' ? BAD BAD! */
-	  fprintf(mfp,"to %s\n",buf);
+	  fprintf(mfp,"to <%s>\n",buf);
 	} else {
 	  /*  Cc: some-address  */
-	  fprintf(mfp,"to %s\n",buf);
+	  fprintf(mfp,"to <%s>\n",buf);
 	}
 }
 
@@ -199,7 +198,7 @@ reporterrs(cfpi)
 	char path[MAXPATHLEN], buf[BUFSIZ], *midbuf, *cp, *s, *eaddr;
 	char *deliveryform;
 	FILE *errfp, *fp;
-	char *boundarystr = (char*)buildboundary("dlvryerr");
+	char *boundarystr;
 	char *notary;
 	struct not *notaries = NULL;
 	char *envid;
@@ -213,6 +212,9 @@ reporterrs(cfpi)
 		return;
 	if (cfpi->erroraddr == NULL)
 		return;
+
+	boundarystr = (char*)buildboundary("dlvryerr");
+
 	eaddr = cfpi->erroraddr;
 	deliveryform = cfpi->deliveryform;
 	envid = cfpi->envid;
@@ -250,11 +252,14 @@ reporterrs(cfpi)
 	  if (!(*cp == _CF_DIAGNOSTIC && *++cp == _CFTAG_NORMAL))
 	    continue;
 	  if (!wroteheader) {
-	    if (*eaddr == '|' || *eaddr == '/')
+	    if (*eaddr == '|' /* Nobody's address starts with a pipe ? */ ||
+		(*eaddr == '/' &&
+		 (strchr(eaddr,'@') == NULL ||
+		  strchr(eaddr,'=') == NULL /*Smells of X.400*/)))
 	      eaddr = "postmaster"; /* Paranoid, eh ? */
 	    fprintf(errfp, "channel error\n");
 
-	    fprintf(errfp, "to %s\n", eaddr);
+	    fprintf(errfp, "to <%s>\n", eaddr);
 	    sprintf(path, "%s/%s/%s", mailshare, FORMSDIR,
 		    deliveryform ? deliveryform : "delivery");
 	    if ((fp = fopen(path, "r")) != NULL) {
