@@ -30,6 +30,7 @@ static void __free(ptr,file,line) void *ptr; char*file;int line; { fprintf(stder
 /* agenda.c */
 extern time_t qipcretry;
 extern int doagenda __((void));
+extern void turnme __((char *));
 
 /* conf.c */
 extern char *qlogdir;
@@ -116,14 +117,13 @@ extern void feed_child      __((struct procinfo *cpidp));
 extern int flush_child      __((struct procinfo *cpidp));
 extern void idle_child      __((struct procinfo *cpidp));
 extern int  start_child     __((struct vertex *vhead, struct web *channel, struct web *host));
-extern SIGNAL_TYPE sig_chld __((int sig));
+extern RETSIGTYPE sig_chld __((int sig));
 extern int mux __((time_t timeout));
 extern void queryipccheck __((void));
 extern void queryipcinit __((void));
 #if defined(USE_BINMKDIR) || defined(USE_BINRMDIR)
 extern int system __((char*));
 #endif
-extern SIGNAL_TYPE sig_chld __((int));
 
 /* update.c */
 extern void update __((int, char *));
