@@ -22,9 +22,24 @@
 #include <fcntl.h>
 #include "mail.h"
 #include <string.h>
-#include NDIR_H
+
+#ifdef HAVE_DIRENT_H
+# include <dirent.h>
+#else /* not HAVE_DIRENT_H */
+# define dirent direct
+# ifdef HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif /* HAVE_SYS_NDIR_H */
+# ifdef HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif /* HAVE_SYS_DIR_H */
+# ifdef HAVE_NDIR_H
+#  include <ndir.h>
+# endif /* HAVE_NDIR_H */
+#endif /* HAVE_DIRENT_H */
+
 #include "zmsignal.h"
-#ifdef USE_MMAP
+#ifdef HAVE_MMAP
 #include <sys/mman.h>
 #endif
 
@@ -95,10 +110,10 @@ static int  lockverify __((struct ctlfile *, char *));
 static int  globmatch   __((char *, char*));
 static void vtxdo   __((struct vertex *, struct config_entry *, char *));
 
-static SIGNAL_TYPE sig_exit   __((int sig));
-static SIGNAL_TYPE sig_alarm  __((int sig));
-static SIGNAL_TYPE sig_iot    __((int sig));
-static SIGNAL_TYPE sig_readcf __((int sig));
+static RETSIGTYPE sig_exit   __((int sig));
+static RETSIGTYPE sig_alarm  __((int sig));
+static RETSIGTYPE sig_iot    __((int sig));
+static RETSIGTYPE sig_readcf __((int sig));
 
 extern char *Version;
 extern char *optarg;
@@ -119,7 +134,7 @@ static int timeserver_pid = 0;
 extern time_t mytime __((time_t *));
 static void init_timeserver __((void));
 
-#ifdef USE_TIMEVAL
+#ifdef HAVE_SELECT /* Well, not exactly kosher assumption.. */
 extern int gettimeofday();
 #else
 extern time_t time __((time_t *));
@@ -335,7 +350,7 @@ main(argc, argv)
 	  if (loginit(SIGHUP) < 0) /* do setlinebuf() there */
 	    die(1, "log initialization failure");
 	  /* close and reopen log files */
-	  SIGNAL_HANDLE(SIGHUP, (SIGNAL_TYPE(*)())loginit);
+	  SIGNAL_HANDLE(SIGHUP, (RETSIGTYPE(*)())loginit);
 	} else {
 	  SIGNAL_IGNORE(SIGHUP); /* no surprises please */
 	  setvbuf(stdout, (char *)NULL, _IOLBF, 0);
@@ -444,9 +459,8 @@ main(argc, argv)
 	SIGNAL_HANDLE(SIGALRM, sig_alarm);	/* process agenda */
 	SIGNAL_HANDLE(SIGUSR2, sig_iot);	/* dump queue info */
 
-#ifdef USE_MMAP /* Either this, or shared segment, or thread.. */
+	/* call it to create the timeserver -- if possible */
 	init_timeserver();
-#endif
 
 	queryipcinit();
 
@@ -554,7 +568,7 @@ main(argc, argv)
 	return 0;
 }
 
-static SIGNAL_TYPE sig_exit(sig)
+static RETSIGTYPE sig_exit(sig)
 int sig;
 {
 	
@@ -567,7 +581,7 @@ int sig;
 	mustexit = 1;
 }
 
-static SIGNAL_TYPE sig_alarm(sig)
+static RETSIGTYPE sig_alarm(sig)
 int sig;
 {
 	gotalarm = 1;
@@ -575,7 +589,7 @@ int sig;
 }
 
 #ifdef SIGUSR2
-static SIGNAL_TYPE sig_iot(sig)
+static RETSIGTYPE sig_iot(sig)
 int sig;
 {
 	dumpq = 1;
@@ -584,7 +598,7 @@ int sig;
 #endif
 
 #ifdef SIGUSR1
-static SIGNAL_TYPE sig_readcf(sig)
+static RETSIGTYPE sig_readcf(sig)
 int sig;
 {
 	rereadcf = 1;
@@ -623,7 +637,7 @@ static int dirqueuescan(dir, dq)
 	struct dirqueue *dq;
 {
 	DIR *dirp;
-	struct NDIR_TYPE *dp;
+	struct dirent *dp;
 	struct stat stbuf;
 	char file[MAXNAMLEN+1];
 	int newents = 0;
@@ -952,6 +966,9 @@ slurp(fd, ino)
 		/* cfp->logident = strsave(s+2); */
 	      } else if (*s == _CF_ERRORADDR) {
 		/* cfp->erroraddr = strsave(s+2); */
+	      } else if (*s == _CF_TURNME) {
+		/* Umm... it is a bit complex */
+		printf("A TURNME request for target '%s'\n",s+2);
 	      }
 	    }
 	  }
@@ -1209,27 +1226,13 @@ static struct ctlfile *vtxprep(cfp)
 	      deletemsg(cp+2, cfp);
 	      break;
 	    case _CF_VERBOSE:
-#ifdef	USE_SETREUID
+
 	      setreuid(0, cfp->uid);
-#else	/* SVID ?? */
-#ifdef	USE_SETEUID
-	      seteuid(cfp->uid);
-#else
-	      /* Only reversible under recent SVID! */
-	      setuid(cfp->uid);
-#endif	/* !USE_SETEUID */
-#endif	/* !USE_SETREUID */
+
 	      cfp->vfp = fopen(cp+2, "a+");
-#ifdef	USE_SETREUID
+
 	      setreuid(0, 0);
-#else	/* SVID ?? */
-#ifdef	USE_SETEUID
-	      seteuid(0);
-#else
-	      /* Only works under recent SVID! */
-	      setuid(0);
-#endif	/* !USE_SETEUID */
-#endif	/* !USE_SETREUID */
+
 	      if (cfp->vfp != NULL) {
 		fseek(cfp->vfp, (long)0, 2);
 		setvbuf(cfp->vfp, (char *)NULL,	_IOLBF, 0);
@@ -1239,6 +1242,11 @@ static struct ctlfile *vtxprep(cfp)
 			"scheduler processing %s\n",
 			cfp->mid);
 	      break;
+	    case _CF_TURNME:
+	      printf("TURNME: %s\n",cp+2);
+	      strlower(cp+2);
+	      turnme(cp+2);
+	      break;
 	    }
 	  }
 	}
@@ -1686,39 +1694,55 @@ timestring()
 	return timebuf;
 }
 
-#ifdef USE_MMAP
+#if defined(HAVE_MMAP)
 struct timeserver {
 	int	pid;
-#ifdef USE_TIMEVAL
+#ifdef HAVE_SELECT
 	struct timeval tv;
 #else
 	time_t	time_sec;
 #endif
+#define MAPSIZE 16*1024
 };
 struct timeserver *timeserver_segment = NULL;
-void init_timeserver()
+static void init_timeserver()
 {
 	int ppid;
 
-#ifndef MAP_ANONYMOUS /* must have a file ? (SunOS 4.1) */
+#if !defined(MAP_ANONYMOUS) || defined(__linux__)
+	/* must have a file ? (SunOS 4.1, et.al.) */
+	int fd = -1;
 	char blk[1024];
 	int i;
 	FILE *fp = tmpfile();
-	if (!fp) return; /* Brr! */ 
+
+	if (fp) {
+	  fd = dup(fileno(fp));
+	  fclose(fp);
+	}
+	if (fd < 0) return; /* Brr! */ 
 
 	memset(blk,0,sizeof(blk));
-	for (i=0; i < 64; ++i)
-	  write(fileno(fp),blk,sizeof(blk));
+	for (i=0; i < MAPSIZE; i += sizeof(blk))
+	  write(fd,blk,sizeof(blk));
+	lseek(fd,0,0);
 
-	timeserver_segment = mmap(NULL, 65536, PROT_READ|PROT_WRITE,
-				  MAP_SHARED, fileno(fp), 0);
-	fclose(fp);
+#ifndef MAP_FILE
+# define MAP_FILE 0 /* SunOS 4.1 does not have this */
+#endif
+	timeserver_segment = (void*)mmap(NULL, MAPSIZE, PROT_READ|PROT_WRITE,
+					 MAP_FILE|MAP_SHARED, fd, 0);
+	close(fd);
 	
 #else
+#ifndef MAP_VARIABLE
+# define MAP_VARIABLE 0 /* Some system have MAP_ANONYMOUS, but
+			   no MAP_VARIABLE.. */
+#endif
 	/* This MAP_ANONYMOUS does work at DEC OSF/1 ... */
-	timeserver_segment = mmap(NULL, 65536, PROT_READ|PROT_WRITE,
-				  MAP_VARIABLE|MAP_ANONYMOUS|MAP_SHARED,
-				  -1, 0);
+	timeserver_segment = (void*)mmap(NULL, MAPSIZE, PROT_READ|PROT_WRITE,
+					 MAP_VARIABLE|MAP_ANONYMOUS|MAP_SHARED,
+					 -1, 0);
 #endif
 	if (-1L == (long)timeserver_segment
 	    || timeserver_segment == NULL) {
@@ -1738,14 +1762,14 @@ void init_timeserver()
 	ppid = getppid(); /* who is our parent ? */
 
 	for(;;) {
-#ifdef USE_SELECT
+#ifdef HAVE_SELECT
 	  struct timeval tv;
 	  int rc;
 
 	  tv.tv_sec = 1;
 	  tv.tv_usec = 0;
 
-	  gettimeofday(&timeserver_segment->tv,NULL);
+	  gettimeofday(&timeserver_segment->tv, NULL);
 
 	  rc = select(0,NULL,NULL,NULL,&tv);
 #else
@@ -1758,15 +1782,21 @@ void init_timeserver()
 	}
 	_exit(0);
 }
+#else
+static void init_timeserver()
+{
+  /* Do nothing.. */
+}
+
 #endif
 
 time_t mytime(timep)
 time_t *timep;
 {
-#ifdef USE_MMAP
+#if defined(HAVE_MMAP)
 	if (timeserver_pid) {
 	  time_t t;
-#ifdef USE_TIMEVAL
+#ifdef HAVE_SELECT
 	  t = timeserver_segment->tv.tv_sec;
 #else
 	  t = timeserver_segment->time_sec;
