@@ -47,11 +47,11 @@ extern int wait();
 # endif
 #endif
 
-#ifndef WIFEXITED
-# define WIFEXITED(s) (((s) >> 8) & 0377)
+#ifndef WEXITSTATUS
+# define WEXITSTATUS(s) (((s) >> 8) & 0377)
 #endif
-#ifndef WIFSIGNALED
-# define WIFSIGNALED(s) ((s) & 0177)
+#ifndef WSIGNALSTATUS
+# define WSIGNALSTATUS(s) ((s) & 0177)
 #endif
 
 struct procinfo *cpids = NULL;
@@ -1002,8 +1002,8 @@ int signum;
 #endif
 	  if (pid <= 0) break;
 
-	  if (WIFEXITED(statloc)) ok = 1;
-	  if (WIFSIGNALED(statloc)) ok = 1;
+	  if (WEXITSTATUS  (statloc) != 0) ok = 1;
+	  if (WSIGNALSTATUS(statloc) != 0) ok = 1;
 
 	  if (verbose)
 	    fprintf(stderr,"sig_chld() pid=%d, ok=%d, stat=0x%x\n",
