@@ -4,11 +4,13 @@
  * By Matti Aarnio <mea@utu.fi> Dec-94
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdio.h>
 #include <sys/types.h>
 #include <errno.h>
 extern int errno;
-#include "libsupport.h"
 #ifndef NO_LOADAVER
 
 /* ================================================================ */
