@@ -17,7 +17,7 @@
 #include "malloc.h"
 
 #ifdef	HAVE_RESOLVER
-#include <netdb.h>
+#include "netdb.h"
 #include <sys/socket.h>
 #include <netinet/in.h>
 #endif
@@ -28,8 +28,8 @@
 #endif	/* TRY_AGAIN */
 #ifdef	BIND
 #undef NOERROR /* Solaris  <sys/socket.h>  has  NOERROR too.. */
-#include <arpa/nameser.h>
-#include <resolv.h>
+#include "arpa/nameser.h"
+#include "resolv.h"
 
 #ifndef	BIND_VER
 #ifdef	GETLONG
@@ -401,7 +401,7 @@ hold(s, errmsgp)
 	if (colon == NULL)
 	  return EX_PROTOCOL;	/* invalid hold condition */
 
-	symid = symbol(s);
+	symid = symbol((void*)s);
 	if (spt_hash == NULL)
 	  spt_hash = sp_init();
 	if ((spl = sp_lookup(symid, spt_hash)) != NULL) {
@@ -669,15 +669,15 @@ hold_script(command)
 	fclose(errfp);
 	close(in[0]);
 
-        if (WIFEXITED(status) > 0) {
+        if (status & 0177) { /* any signals ? */
 	  if (cp != errormsg)
 	    *cp++ = ' ';
-	  sprintf(cp, "[signal %d", WIFEXITED(status) & 127);
-	  if (WIFEXITED(status)&0200)
+	  sprintf(cp, "[signal %d", status & 0177);
+	  if (status & 0200)
 	    strcat(cp, " (Core dumped)");
 	  strcat(cp, "]");
 	  return 0;
-        } else if (WEXITSTATUS(status) > 0) {
+        } else if (WEXITSTATUS(status) != 0) {
 	  if (cp != errormsg)
 	    *cp++ = ' ';
 	  sprintf(cp, "[exit status %d]", WEXITSTATUS(status));
