@@ -17,7 +17,7 @@
  * rewritten, and those do tack together for each recipients (rp->)
  * There
  *    rp->newmsgheader    is a pointer to an element on  rp->desc->headers[]
- *    rp->newmsgheadercvt is prespectively an elt on  rp->desc->headerscvt[]
+ *    rp->newmsgheadercvt is respectively an elt on  rp->desc->headerscvt[]
  *
  * The routine-collection   mimeheaders.c  creates converted headers,
  * if the receiving system needs them. Converted data is created only
@@ -27,9 +27,8 @@
  * [mea@utu.fi] - 25-Jul-94
  */
 
-
-#include <stdio.h>
 #include "hostenv.h"
+#include <stdio.h>
 #include <ctype.h>
 #include <sys/stat.h>
 #include <sys/file.h>
@@ -41,7 +40,7 @@
 #include "mail.h"
 #include "malloc.h"
 #include "ta.h"
-#ifdef	USE_MMAP
+#ifdef	HAVE_MMAP
 #include <sys/mman.h>
 #endif
 
@@ -103,7 +102,7 @@ ctlclose(dp)
 	    continue;
 	  diagnostic(rp, EX_TEMPFAIL, 0, "address was left locked!!");
 	}
-#ifdef	USE_MMAP
+#ifdef	HAVE_MMAP
 	if (dp->let_buffer != NULL)
 	  munmap(dp->let_buffer, dp->let_end - dp->let_buffer);
 	if (dp->ctlmap != NULL)
@@ -141,6 +140,14 @@ ctlclose(dp)
 	}
 	free(dp->msgheaders);
 	dp->msgheaders = NULL; /* XX: Excessive caution.. */
+	for (msghpp = dp->msgheaderscvt; msghpp &&  *msghpp; ++msghpp) {
+	  char **msghp = *msghpp;
+	  for ( ; msghp && *msghp ; ++msghp )
+	    free(*msghp); /* These CVTs are always malloc()ed strings */
+	  free(*msghpp);
+	}
+	free(dp->msgheaderscvt);
+	dp->msgheaders = NULL; /* XX: Excessive caution.. */
 }
 
 
@@ -221,10 +228,10 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	  char cwd[MAXPATHLEN], buf[MAXPATHLEN+MAXPATHLEN+100];
 	  int e = errno;	/* Save it over the getwd() */
 
-#ifdef	USE_GETWD
-	  getwd(cwd);
-#else
+#ifdef	HAVE_GETCWD
 	  getcwd(cwd,MAXPATHLEN);
+#else
+	  getwd(cwd);
 #endif
 	  sprintf(buf,
 		  "Cannot open control file \"%%s\" from \"%s\" for \"%%s/%%s\" as uid %d! (%%m)",
@@ -252,7 +259,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	  warning("Out of virtual memory!", (char *)NULL);
 	  exit(EX_SOFTWARE);
 	}
-#ifdef USE_MMAP
+#ifdef HAVE_MMAP
 #ifndef MAP_VARIABLE
 # define MAP_VARIABLE 0
 #endif
@@ -264,7 +271,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 				PROT_READ|PROT_WRITE,
 				MAP_FILE|MAP_SHARED|MAP_VARIABLE,
 				d.ctlfd, 0);
-	if ((int)d.ctlmap == (int)-1L)
+	if ((long)d.ctlmap == -1L)
 	  d.ctlmap = NULL; /* Failed ?? */
 #else
 	d.ctlmap = NULL;
@@ -288,7 +295,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	  free(d.contents);
 	  free((char *)d.offset);
 	  close(d.ctlfd);
-#ifdef USE_MMAP
+#ifdef HAVE_MMAP
 	  if (d.ctlmap != NULL)
 	    munmap(d.ctlmap, d.contentsize);
 #endif	  
@@ -559,11 +566,11 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	  close(d.msgfd);
 	  return NULL;
 	}
-#ifdef	USE_MMAP
+#ifdef	HAVE_MMAP
 	d.let_buffer = (char *)mmap(NULL, stbuf.st_size, PROT_READ,
 				    MAP_FILE|MAP_SHARED|MAP_VARIABLE,
 				    d.msgfd, 0);
-	if ((int)d.let_buffer == (int)-1L) {
+	if ((long)d.let_buffer == -1L) {
 	  warning("Out of MMAP() memory! Tried to map in (r/o) %d bytes (%m)",
 		  stbuf.st_size);
 	  free(mfpath);
