@@ -1,6 +1,6 @@
-#include <stdio.h>
 #include "hostenv.h"
-#ifdef linux
+#include <stdio.h>
+#ifdef __linux__
 #define __USE_BSD 1
 #endif
 #include <ctype.h>
@@ -10,23 +10,22 @@
 #include <sys/stat.h>
 #include "zmsignal.h"
 #include "malloc.h"
-#include NDIR_H
 
-#if	defined(USE_HOSTS) || defined(USE_RESOLV)
-#include <netdb.h>
+#if	defined(HAVE_RESOLVER)
+#include "netdb.h"
 #include <sys/socket.h>
 #include <netinet/in.h>
-#endif	/* USE_HOSTS || USE_RESOLV */
+#endif	/* HAVE_RESOLVER */
 
 #include "ta.h"
 
-#if	defined(TRY_AGAIN) && defined(USE_RESOLV)
+#if	defined(TRY_AGAIN) && defined(HAVE_RESOLVER)
 #define	BIND		/* Want BIND (named) nameserver support enabled */
 #endif	/* TRY_AGAIN */
 #ifdef	BIND
 #undef NOERROR /* Solaris  <sys/socket.h>  has  NOERROR too.. */
-#include <arpa/nameser.h>
-#include <resolv.h>
+#include "arpa/nameser.h"
+#include "resolv.h"
 
 #ifndef	BIND_VER
 #ifdef	GETLONG
