@@ -5,6 +5,7 @@
  *	(and   Markku T Jarvinen <mta@sci.fi>)
  */
 
+#include "hostenv.h"
 #include <varargs.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -27,7 +28,7 @@ extern void *realloc();
 
 int cvtspace_copy __((struct rcpt *rp));
 
-#undef USE_NEW_MIME2_CODE /* not yet ready.. */
+/* #define USE_NEW_MIME2_CODE /* TESTING! */ /* not yet ready.. */
 #ifdef USE_NEW_MIME2_CODE
 
 /*
@@ -45,64 +46,8 @@ const char *Base16Code = "0123456789ABCDEF";
 # define MAX(i1,i2) ((i1 < i2) ? i2 : i1)
 #endif
 
-/*
- * go the line word by word and return the result
- * line will be split on whitespace if possible
- */
 
 static int
-header8bit2QP(line, defcharset, outp, sizep, fold)
-     unsigned char *line;
-     const char *defcharset;
-     unsigned char **outp;
-     int *sizep;
-     int fold;
-{
-	unsigned char *ptr, *outptr, *outline;
-	unsigned char tmp[MAX(MAXLINE,BUFSIZ)];
-	unsigned char word[MAXENCODEDLINE+2];
-	int len=0, abslen=0, lines=0, linelen=0, inside=0, hdr_offset=1;
-	int fffppp;
-  
-	linelen = strlen(line);
-	ptr = line;
-	if (verboselog)
-	  fprintf(verboselog,"header8bit2QP: QP\n");
-	sprintf((void*)tmp, "=?%s?Q?", defcharset);
-#ifndef TRUSTFORMAT
-	while (*ptr != '\0' && *ptr != ':') {
-	  ptr++;
-	  hdr_offset++;
-	}
-	if (*ptr == '\0')
-	  hdr_offset = 1;
-	else
-	  hdr_offset++;
-	ptr = line;
-#endif
-	while (*ptr != '\0') {
-	  if (header8bit2QPnextword(&ptr, word, &len, &inside, tmp, fold, hdr_offset)) {
-	    lines++;
-	  }
-	  abslen += strlen(word);
-	  if (abslen >= *sizep) {
-	    *outp = realloc(*outp,abslen+2);
-	    *sizep = abslen+2;
-	  }
-	  strcat(*outp, word);
-	}
-#if 0
-	{
-	  int fffppp = open("/tmp/ss", 256+8+1, 0777);
-	  write(fffppp, line, strlen(line));
-	  write(fffppp, out, strlen(out));
-	  close(fffppp);
-	}
-#endif
-	return 1;
-}
-
-int
 header8bit2QPnextword(line, word, len, inside, start_encode, fold, hdr_offset)
      unsigned char **line;
      unsigned char *word;
@@ -183,6 +128,63 @@ header8bit2QPnextword(line, word, len, inside, start_encode, fold, hdr_offset)
 	*tmp = '\0';
 	return 0;
 }	
+
+/*
+ * go the line word by word and return the result
+ * line will be split on whitespace if possible
+ */
+
+static int
+header8bit2QP(line, defcharset, outp, sizep, fold)
+     unsigned char *line;
+     const char *defcharset;
+     unsigned char **outp;
+     int *sizep;
+     int fold;
+{
+	unsigned char *ptr, *outptr, *outline;
+	unsigned char tmp[MAX(MAXLINE,BUFSIZ)];
+	unsigned char word[MAXENCODEDLINE+2];
+	int len=0, abslen=0, lines=0, linelen=0, inside=0, hdr_offset=1;
+	int fffppp;
+  
+	linelen = strlen(line);
+	ptr = line;
+	if (verboselog)
+	  fprintf(verboselog,"header8bit2QP: QP\n");
+	sprintf((void*)tmp, "=?%s?Q?", defcharset);
+#ifndef TRUSTFORMAT
+	while (*ptr != '\0' && *ptr != ':') {
+	  ptr++;
+	  hdr_offset++;
+	}
+	if (*ptr == '\0')
+	  hdr_offset = 1;
+	else
+	  hdr_offset++;
+	ptr = line;
+#endif
+	while (*ptr != '\0') {
+	  if (header8bit2QPnextword(&ptr, word, &len, &inside, tmp, fold, hdr_offset)) {
+	    lines++;
+	  }
+	  abslen += strlen(word);
+	  if (abslen >= *sizep) {
+	    *outp = realloc(*outp,abslen+2);
+	    *sizep = abslen+2;
+	  }
+	  strcat(*outp, word);
+	}
+#if 0
+	{
+	  int fffppp = open("/tmp/ss", 256+8+1, 0777);
+	  write(fffppp, line, strlen(line));
+	  write(fffppp, out, strlen(out));
+	  close(fffppp);
+	}
+#endif
+	return 1;
+}
 #endif
 
 /* This is rather half-baked approach, it creates correct tokens,
@@ -262,10 +264,13 @@ if(verboselog)fprintf(verboselog,"After processing: '%s'\n",newbuf);
 	/* New MIME-2 code by mta@sci.fi in use.. */
 
 	char **inhdr, **inhdr1;
+	char **outhdr;
+	int   outcnt = 0;
 	char *sumstr = emalloc(1);
 	int   sumlen = 0;
 
 	*sumstr = 0;
+	outhdr = NULL;
 
 	if (*(rp->newmsgheadercvt) == NULL)
 	  if (!cvtspace_copy(rp))
@@ -287,11 +292,75 @@ if(verboselog)fprintf(verboselog,"After processing: '%s'\n",newbuf);
 	    sumlen += thislen+1;
 	    sumstr = nn;
 	  } else {
-	    if (*sumstr != 0) {
+	    /* Not a continuation line, Some sort of 'first line' */
+	    if (sumlen > 0) {
+	      char *s, *p;
+	      int rc, size = 0;
+	      char *outstr = emalloc(sumlen);
+
+	      rc = header8bit2QP(sumstr,DEFCHARSET,&outstr,&size,1);
+	      s = outstr;
+	      while (s && *s) {
+		p = strlen(s,'\n');
+		if (p) *p++ = 0;
+		size = strlen(s);
+		if (outhdr == NULL)
+		  outhdr = (char **)emalloc(sizeof(void*) * 2);
+		else
+		  outhdr = (char**)erealloc(outhdr,
+					    sizeof(void*) * (outcnt+2));
+		outhdr[outcnt] = (char*)emalloc(size+1);
+		strcpy(outhdr[outcnt],s);
+		++outcnt;
+		s = p;
+	      }
+
+	      /* Clean up the results */
+	      sumlen = 0;
+	      free(outstr);
+	    } /* sumlen > 0 */
+
+	    /* Collect the 'first line' */
+	    nn = realloc(sumstr,thislen+2);
+	    if (!nn) return -1; /* AUTCH! */
+	    strcpy(sumstr,*inhdr);
+	    sumlen = thislen;
 
-	    }
 	  }
 	}
+
+	/* Process the last of the headers */
+
+	if (sumlen > 0) {
+	  char *s, *p;
+	  int rc, size = 0;
+	  char *outstr = emalloc(sumlen);
+
+	  rc = header8bit2QP(sumstr,DEFCHARSET,&outstr,&size,1);
+	  s = outstr;
+	  while (s && *s) {
+	    p = strlen(s,'\n');
+	    if (p) *p++ = 0;
+	    size = strlen(s);
+	    if (outhdr == NULL)
+	      outhdr = (char **)emalloc(sizeof(void*) * 2);
+	    else
+	      outhdr = (char**)erealloc(outhdr,
+					sizeof(void*) * (outcnt+2));
+	    outhdr[outcnt] = (char*)emalloc(size+1);
+	    strcpy(outhdr[outcnt],s);
+	    ++outcnt;
+	    s = p;
+	  }
+
+	  /* Clean up the results */
+	  sumlen = 0;
+	  free(outstr);
+	} /* sumlen > 0 */
+	free(sumstr);
+
+	/* XX: replace the converted headers with  outhdr -set. */
+	    /* XX: resize (enlarge) the header array if need be */	
+
 #endif /* USE_NEW_MIME2_CODE */
 }
-
