@@ -25,6 +25,7 @@
  * [mea@utu.fi] - 25-Jul-94
  */
 
+#include "hostenv.h"
 #include <varargs.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -377,7 +378,7 @@ char **ct_linep;	/* Could be multiline! */
 	  /* Check for possible parameters on the first and/or continuation
 	     line(s) of the header line... */
 	  char paramname[40];
-	  char *parval;
+	  char *parval, c;
 
 	  if (*s == 0) {
 	  /* Check if we have a continuation line */
@@ -396,9 +397,10 @@ char **ct_linep;	/* Could be multiline! */
 	  if (*s == 0) continue; /* last token on this line ? */
 
 	  p = s;
-	  while (*s && isascii(*s) && (isalnum(*s) || *s == '-' ||
-				       *s == '_'))
-	    ++s; /* Scan over a parameter name.. */
+	  c = *s;
+	  while (c && (('A' <= c && c <= 'Z') || ('a' <= c && c <= 'z') ||
+		       ('0' <= c && c <= '9') || c == '-' || c == '_'))
+	    c = *++s; /* Scan over a parameter name.. */
 
 	  /* Ok, `p' points to a parameter name string */
 	  if (s < (p+sizeof(paramname))) {
@@ -430,8 +432,31 @@ char **ct_linep;	/* Could be multiline! */
 	      while (*s && *s != ';' && *s != ' ' && *s != '\t')
 		++s;
 	    }
-	  } /* else
-	       -- XX: what if no `=' after the parameter name ? */
+	  } else {
+	    /* XX: what if no `=' after the parameter name ? */
+	    /* ... it never happens, and then somebody screw things
+	       up in 9th of June, 1996, and nic.funet.fi experienced
+	       smtp crash.. */
+	    /* pv is at the start of the stuff as is.. */
+	    ++s;
+	    if (*s == '"') {
+	      /* Scan a quoted string, stop at trailing '"' */
+	      int quoted = 0; /* Quoted with '\' */
+	      ++s;
+	      while (*s) {
+		if (!quoted && *s == '"') break;
+		if (*s == '\\') /* This quote skips over the next char */
+		  quoted = 2;
+		if (quoted) --quoted;
+		++s;
+	      }
+	      if (*s) ++s; /* Skip the trailing '"' */
+	    } else {
+	      /* Scan an alphanumeric string -- stop at ';', and LWSP */
+	      while (*s && *s != ';' && *s != ' ' && *s != '\t')
+		++s;
+	    }
+	  }
 
 	  parval = emalloc((s - pv) + 2);
 	  ss = parval;
