@@ -10,7 +10,7 @@
  * By Ken Lalonde, based loosely on code from mush and pine.
  */
 
-#ifdef USE_DOTLOCK	/* Well, we DO use DOTLOCK scheme! */
+#if defined(HAVE_DOTLOCK) /* Well, we DO use DOTLOCK scheme! */
 
 /*
    #include <sys/types.h>
