@@ -25,6 +25,9 @@
 #include <fcntl.h>
 #include <sys/file.h>
 #include <sys/stat.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h> /* F_LOCK is there at some systems.. */
+#endif
 #include "mail.h"
 #include "zmsignal.h"
 #include "ta.h"
@@ -973,7 +976,7 @@ deliver(dp, rp, timestring)
 	    return;
 	  }
 #endif	/* USE_NFSMBOX */
-#ifdef	HAVE_LOCKF
+#if defined(HAVE_LOCKF) && defined(F_LOCK)
 	  /* Seek to begining for locking! [mea@utu.fi] */
 	  if (lseek(fdmail,0L,L_SET) < 0 ||
 	      lockf(fdmail, F_LOCK, 0) < 0) {
@@ -1036,7 +1039,7 @@ deliver(dp, rp, timestring)
 #ifdef	USE_NFSMBOX
 	  unlock(file);
 #endif	/* USE_NFSMBOX */
-#ifdef	HAVE_LOCKF
+#if defined(HAVE_LOCKF) && defined(F_ULOCK)
 	  lseek(fdmail,0L,L_SET);
 	  lockf(fdmail, F_ULOCK, 0);/* XX: does this really work?*/
 	  /* Yes it does, but it needs seek to begining of
