@@ -35,11 +35,11 @@ extern int wait();
 # endif
 #endif
 
-#ifndef WIFEXITED
-# define WIFEXITED(s) (((s) >> 8) & 0377)
+#ifndef WEXITSTATUS
+# define WEXITSTATUS(s) (((s) >> 8) & 0377)
 #endif
-#ifndef WIFSIGNALED
-# define WIFSIGNALED(s) ((s) & 0177)
+#ifndef WSIGNALSTATUS
+# define WSIGNALSTATUS(s) ((s) & 0177)
 #endif
 
 #ifndef	L_SET
@@ -395,7 +395,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	}
 	av[i] = NULL;
 	/* now we can fork off and run the command... */
-	if (pipe(in) < 0) {
+	if (pipe(out) < 0) {
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
 	    notaryreport(rp->addr->user,"failed",
 			 "4.3.0 (Out of system resources, pipe creation failed)",
@@ -406,7 +406,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  }
 	  return;
 	}
-	if (pipe(out) < 0) {
+	if (pipe(in) < 0) {
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
 	    notaryreport(rp->addr->user,"failed",
 			 "4.3.0 (Out of system resources, pipe creation failed)",
@@ -418,19 +418,30 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  return;
 	}
 	if ((pid = fork()) == 0) { /* child, run the command */
-	  close(in[0]);
-	  close(out[1]);
 	  if (!(mp->flags & MO_NORESETUID))
 	    setuid(getuid());
-	  /* its stdout and stderr is the pipe, its stdin is our tafp */
-	  close(0);
-	  close(1);
-	  close(2);
-	  dup2(out[0], 0);
-	  close(out[0]);
-	  dup2(in[1], 1);
-	  dup2(in[1], 2);
-	  close(in[1]);
+	  if (in[1] > 0) {
+	    /* its stdout and stderr is the pipe, its stdin is our tafp */
+	    if (out[0] > 0) dup2(out[0], 0);
+	    if (in[1]  > 1) dup2(in[1], 1);
+	    dup2(1, 2);
+	    if (in[0] > 2) close(in[0]);
+	    if (in[1] > 2) close(in[1]);
+	    if (out[0] > 2) close(out[0]);
+	    if (out[1] > 2) close(out[1]);
+	  } else {
+	    close(in[0]);
+	    close(out[1]);
+	    /* its stdout and stderr is the pipe, its stdin is our tafp */
+	    close(0);
+	    close(1);
+	    close(2);
+	    dup2(out[0], 0);
+	    close(out[0]);
+	    dup2(in[1], 1);
+	    dup2(in[1], 2);
+	    close(in[1]);
+	  }
 	  execv(mp->command, av);
 	  _exit(254);
 	} else if (pid < 0) {	/* couldn't fork, complain */
@@ -563,47 +574,53 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  *cp = '\0';
 	fclose(errfp);
 	close(in[0]);	/* more paranoia */
-	pid = wait(&status);
 	cp = buf + strlen(buf);
-	  if (WIFSIGNALED(status)) {
-	    if (cp != buf)
-	      *cp++ = ' ';
-	    sprintf(cp, "[signal %d", WIFSIGNALED(status));
-	    if (status&0200)
-	      strcat(cp, " (Core dumped)");
-	    strcat(cp, "]");
-	    i = EX_TEMPFAIL;
-	  } else if (WIFEXITED(status) == 0 || WIFEXITED(status) == EX_OK) {
-	    i = EX_OK;
-	  } else {
-	    i = WIFEXITED(status);
-	    for (exp = exmap; exp->origstatus != 0; ++exp)
-	      if (exp->origstatus == i)
-		break;
 
-	    s = exp->statusmsg;
-	    i = exp->newstatus;
-	    sprintf(cp, "[exit status %d", WIFEXITED(status));
-	    if (s)
-	      sprintf(cp+strlen(cp), " (%s)", s);
-	    sprintf(cp+strlen(cp), " of command: %s", mp->command);
-	    strcat(cp, "]");
-	  }
-	  if (verboselog)
-	    fprintf(verboselog,"Diagnostic: %s\n",cp);
-	  for (rp = startrp; rp != endrp; rp = rp->next) {
-	    if (i == EX_OK)
-	      notaryreport(rp->addr->user, "relayed",
-			   "2.2.0",
-			   "250 Delivered");
-	    else
-	      notaryreport(rp->addr->user, "failed",
-			   "4.3.0",
-			   "430 Unknown error");
-	    diagnostic(rp, i, 0, "%s", buf);
-	  }
-	  /* XX: still need to deal with MO_STRIPQUOTES */
+	pid = wait(&status);
+	if (WSIGNALSTATUS(status) != 0) {
+	  if (cp != buf)
+	    *cp++ = ' ';
+	  sprintf(cp, "[signal %d", WSIGNALSTATUS(status));
+	  if (status&0200)
+	    strcat(cp, " (Core dumped)");
+	  strcat(cp, "]");
+	  i = EX_TEMPFAIL;
+	} else if (WEXITSTATUS(status) == 0
+#if EX_OK != 0
+		   || WEXITSTATUS(status) == EX_OK
+#endif
+		   ) {
+	  i = EX_OK;
+	} else {
+	  i = WEXITSTATUS(status);
+	  s = NULL;
+	  for (exp = exmap; exp->origstatus != 0; ++exp)
+	    if (exp->origstatus == i) {
+	      s = exp->statusmsg;
+	      i = exp->newstatus;
+	      break;
+	    }
+	  sprintf(cp, "[exit status %d", WEXITSTATUS(status));
+	  if (s)
+	    sprintf(cp+strlen(cp), " (%s)", s);
+	  sprintf(cp+strlen(cp), " of command: %s", mp->command);
+	  strcat(cp, "]");
 	}
+	if (verboselog)
+	  fprintf(verboselog,"Diagnostic: %s\n",cp);
+	for (rp = startrp; rp != endrp; rp = rp->next) {
+	  if (i == EX_OK)
+	    notaryreport(rp->addr->user, "relayed",
+			 "2.2.0",
+			 "250 Delivered");
+	  else
+	    notaryreport(rp->addr->user, "failed",
+			 "4.3.0",
+			 "430 Unknown error");
+	  diagnostic(rp, i, 0, "%s", buf);
+	}
+	/* XX: still need to deal with MO_STRIPQUOTES */
+}
 
 /*
  * appendlet - append letter to file pointed at by fd
