@@ -56,8 +56,8 @@
 #undef NOERROR		/* Several SysV-streams using systems have NOERROR,
 			   which is not the same as  <arpa/nameser.h> has! */
 #endif
-#include <arpa/nameser.h>
-#include <resolv.h>
+#include "arpa/nameser.h"
+#include "resolv.h"
 
 #ifndef	BIND_VER
 #ifdef	GETLONG
@@ -1580,7 +1580,7 @@ smtpopen(host, fparr, verboselog)
 	      fclose(fparr[0]); fparr[0] = NULL;
 	      fclose(fparr[1]); fparr[1] = NULL;
 	      if (logfp)
-		fprintf(logfp, "%s#\t(closed SMTP channel - HELO failed??)\n", logtag());
+		fprintf(logfp, "%s#\t(closed SMTP channel - HELO failed ?)\n", logtag());
 	      smtp_isopen = 0;
 	    }
 	    if (i == EX_TEMPFAIL) {
@@ -1864,8 +1864,13 @@ makeconn(hp, fdp, verboselog, fparr)
 	}
 #endif
 	for (hp_init(hp); *hp_getaddr() != NULL; hp_nextaddr()) {
-	  if (matchmyaddress(*hp_getaddr(),hp->h_length)) {
-	    notaryreport(NULL,FAILED,"5.4.6 (trying to talk with myself!)","smtp; 500 (Trying to talk with myself!)");
+	  int i;
+	  if ((i = matchmyaddress(*hp_getaddr(),hp->h_length))) {
+	    if (i == 2) {
+	      notaryreport(NULL,FAILED,"5.4.6 (trying to talk to loopback (=myself)!)","smtp; 500 (Trying to talk to loopback (=myself)!)");
+	    } else {
+	      notaryreport(NULL,FAILED,"5.4.6 (trying to talk with myself!)","smtp; 500 (Trying to talk with myself!)");
+	    }
 	    sprintf(remotemsg,"Trying to talk with myself!");
 	    break;		/* TEMPFAIL or UNAVAILABLE.. */
 	  }
@@ -1931,6 +1936,8 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 
 	time(&now);
 
+
+
 	strcpy(ipaddress, dottedquad((struct in_addr *)ipaddrcp));
 	if (conndebug)
 	  fprintf(stderr, "Trying %s [%s] ... ", hostname, ipaddress);
@@ -1957,7 +1964,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	  return EX_TEMPFAIL;
 	}
 	if (wantreserved && getuid() == 0) {
-	  for (p = IPPORT_RESERVED-1; p >= IPPORT_RESERVED/2; --p) {
+	  for (p = IPPORT_RESERVED-1; p >= (IPPORT_RESERVED/2); --p) {
 	    sad.sin_port = htons(p);
 	    if (bind(s, (struct sockaddr *)&sad, sizeof sad) >= 0)
 	      break;
@@ -1974,7 +1981,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	      return EX_UNAVAILABLE;
 	    }
 	  }
-	  if (p < IPPORT_RESERVED/2) {
+	  if (p < (IPPORT_RESERVED/2)) {
 	    sprintf(remotemsg, "too many busy ports");
 	    notaryreport(NULL,FAILED,"4.4.0 (internal error, too many busy ports)","smtp; 500 (Internal error, too many busy ports)");
 	    if (conndebug)
@@ -2507,7 +2514,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf, syncrp)
 	  fd_nonblockingmode(infd);
 	  for (;;) {
 	    r = read(infd, buf, sizeof buf);
-	    if (r == 0) break; /* EOF ??!! */
+	    if (r == 0) break; /* EOF ?! */
 	    if (r < 0)
 	      break;
 	    if (pipebuf == NULL) {
@@ -2601,7 +2608,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf, syncrp)
 	  fd_nonblockingmode(infd);
 	  for (;;) {
 	    r = read(infd, buf, sizeof buf);
-	    if (r == 0) break; /* EOF ??!! */
+	    if (r == 0) break; /* EOF ?! */
 	    if (r < 0)
 	      break;
 	    if (pipebuf == NULL) {
@@ -2845,6 +2852,7 @@ stashmyaddresses(host)
 	  memcpy(&myaddrs[(--naddrs)*hp->h_length], *hp_getaddr(), hp->h_length);
 }
 
+
 int
 matchmyaddress(addr,len)
 	char *addr;
@@ -2853,11 +2861,20 @@ matchmyaddress(addr,len)
 	int i;
 
 	if (!myaddrs) return 0; /* Don't know my addresses ! */
+	
 	for (i = 0; i < nmyaddrs; ++i) {
 	  /* if this is myself, skip to next MX host */
 	  if (memcmp(addr, &myaddrs[i*len],len) == 0)
 	    return 1;
 	}
+	/* Also match loopback net.. */
+	if (len == 4) {
+	  unsigned int ia; /* 32 bit address */
+	  memcpy(&ia,addr,4);
+	  if (IN_CLASSA(ntohl(ia)) &&
+	      (ntohl(ia) & IN_CLASSA_NET) == INADDR_LOOPBACK)
+	    return 2;
+	}
 	return 0;
 }
 
@@ -2866,6 +2883,7 @@ matchmyaddresses(hp)
 	struct hostent *hp;
 {
 	char **alist;
+	int i;
 #ifndef	h_addr	/* no h_addr macro -- presumably 4.2BSD or earlier.. */
 	char *fakealist[2];
 
@@ -2876,8 +2894,8 @@ matchmyaddresses(hp)
 	alist = hp->h_addr_list;
 #endif
 	while (alist && *alist) {
-	  if (matchmyaddress(*alist,hp->h_length))
-	    return 1;
+	  if ((i = matchmyaddress(*alist,hp->h_length)))
+	    return i;
 	  ++alist;
 	}
 	return 0;
@@ -3033,8 +3051,8 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	    continue;		/* Well well.. spurious! */
 	  }
 	  if (cistrcmp(hent->h_name, myhostname) == 0
-	      || matchmyaddresses(hent))
-	    if (maxpref < 0 || maxpref > mx[nmx].pref)
+	      || matchmyaddresses(hent) == 1)
+	    if (maxpref < 0 || maxpref > (int)mx[nmx].pref)
 	      maxpref = mx[nmx].pref;
 
 	  if ((mx[nmx].host = (msgdata *)emalloc(strlen((void*)buf)+1)) == NULL) {
@@ -3060,7 +3078,7 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	/* discard MX RRs with a value >= that of  myhost */
 	if (maxpref >= 0) {
 	  for (n = i = 0; n < nmx; ++n) {
-	    if (mx[n].pref >= maxpref) {
+	    if ((int)mx[n].pref >= maxpref) {
 	      free(mx[n].host);
 	      mx[n].host = NULL;
 	      ++i;
